@@ -5,7 +5,8 @@
 ##' @param method character string specifying the method used to calculate the pseudo functional k-nearest neighbor density. Valid options of are 'FADP1' and 'FADP2' (see details in references). The default is 'FADP1'.
 ##' @param proportion numeric, a number or numeric vector of numbers within the range [0,1], specifying to automatically select the smoothing parameter k in density estimation (see details). The default is 0.1, 0.2, ... ,1.
 ##' @param f.cut numeric, a number within the range [0,1], specified to automatically select cluster centroids from the decision plot. The default is 0.15.
-##' @param pve numeric, a number within the range [0,1], the proportion of variance explained: used to choose the number of functional principal components. The default is 0.99. When the method is chosen to be 'FADP1', there is no need to specify parameter 'pve' for univariate functional data clustering.
+##' @param pve numeric, a number within the range [0,1], the proportion of variance explained: used to choose the number of functional principal components. The default is 0.9. When the method is chosen to be 'FADP1', there is no need to specify parameter 'pve' for univariate functional data clustering.
+##' @param stats character string specifying the distance based statistics for cluster validation and determining the number of clusters. Valid options are 'silhouette', 'Dunn', and 'CH' (See the description document of the cluster.stats function in the fpc R package for more details about these statistics). The default is "silhouette".
 ##' @details Given n functional objects or curves, FADPclust() calculates f(x) and delta(x) for each object based on the semi-metric distance (see details in references), where f(x) is the local density calculated by the functional k-nearest neighbor density estimator of curve x, and delta(x) is the shortest semi-metric distance between sample curve x and y for all samples y such that f(x) <= f(y). Functional objects or curves with large f and large delta values are labeled class centroids. In other words, they appear as isolated points in the upper right corner of the f vs delta plot (the decision plot, see details in FADPplot). After cluster centroids are determined, other obejects are clustered according to their semi-metric distances to the closes centroids.
 ##'
 ##' The smoothing parameter k in functional k-nearest neighbor density estimation must be explicitly provided. Following Lauter (1988)'s idea, suggest that the optimal size of k satisfies a certain proportion, k = a*n^(4/5), where a is a parameter about the optimal proportion to be determined. Here, users enters variable 'proportion' to specify the parameter a.
@@ -25,31 +26,37 @@
 ##' \item Lauter, H. (1988), "Silverman, B. W.: "Density Estimation for Statistics and Data Analysis.," Biometrical Journal, 30(7), 876-877.
 ##' \item Wang, X. F., and Xu, Y. (2016), "Fast Clustering Using Adaptive Density Peak Detection," Statistical Methods in Medical Research.
 ##' \item Rodriguez, A., and Laio, A. (2014), "Machine learning. Clustering by fast search and find of density peaks," Science, 344(6191), 1492.
-##' \item Yaohui, L., Zhengming, M., and Fang, Y. (2017), "Adaptive density peak clustering based on K-nearest neighbors with aggregating strategy," Knowledge-Based Systems, 133(oct.1), 208-220.
+##' \item Liu Y, Ma Z, and Yu F. (2017), "Adaptive density peak clustering based on K-nearest neighbors with aggregating strategy," Knowledge-Based Systems, 133(oct.1), 208-220.
 ##' }
 ##' @seealso \code{\link{FADPsummary}}, \code{\link{FADPplot}}.
 ##' @examples
 ##' ###univariate functional data
 ##' data("simData1")
 ##' plot(simData1, xlab = "x", ylab = "y")
-##' FADP1.ans <- FADPclust(fdata = simData1, cluster = 2:10, method = "FADP1",
-##'                        proportion = seq(0.02, 0.2, 0.02))
-##' FADP2.ans <- FADPclust(fdata = simData1, cluster = 2:10, method = "FADP2",
-##'                      proportion = seq(0.02, 0.2, 0.02), pve = 0.9)
+##' FADP1.ans <- FADPclust(fdata = simData1, cluster = 2:5, method = "FADP1",
+##'                        proportion = seq(0.02, 0.2, 0.02), f.cut = 0.15,
+##'                        stats = "silhouette")
 ##' FADPsummary(FADP1.ans); FADPplot(FADP1.ans)
+##' \donttest{
+##' FADP2.ans <- FADPclust(fdata = simData1, cluster = 2:5, method = "FADP2",
+##'                        proportion = seq(0.02, 0.2, 0.02), f.cut = 0.15,
+##'                        pve = 0.9, stats = "silhouette")
 ##' FADPsummary(FADP2.ans); FADPplot(FADP2.ans)
 ##'
-##' \donttest{
 ##' ###multivariate functional data
 ##' data("simData2")
-##' FADP1.ans <- FADPclust(fdata = simData2, cluster = 2:10, method = "FADP1",
-##'                        proportion = seq(0.02, 0.2, 0.02), pve = 0.9)
-##' FADP2.ans <- FADPclust(fdata = simData2, cluster = 2:10, method = "FADP2",
-##'                      proportion = seq(0.02, 0.2, 0.02), pve = 0.9)
+##' FADP1.ans <- FADPclust(fdata = simData2, cluster = 2:5, method = "FADP1",
+##'                        proportion = seq(0.02, 0.2, 0.02), f.cut = 0.15,
+##'                        pve = 0.9, stats = "silhouette")
 ##' FADPsummary(FADP1.ans); FADPplot(FADP1.ans)
+##'
+##' FADP2.ans <- FADPclust(fdata = simData2, cluster = 2:5, method = "FADP2",
+##'                        proportion = seq(0.02, 0.2, 0.02), f.cut = 0.15,
+##'                        pve = 0.9, stats = "silhouette")
 ##' FADPsummary(FADP2.ans); FADPplot(FADP2.ans)
 ##' }
 ##' @import fda
+##' @import fpc
 ##' @import cluster
 ##' @import MFPCA
 ##' @import funData
@@ -58,47 +65,31 @@
 ##' @importFrom fda.usc semimetric.basis
 ##' @export
 
-FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(0.1,1,0.1), f.cut = 0.15, pve = 0.99 )
-{
-  if( class(fdata) != "fd" & class(fdata) != "list" ){
-    stop( "Error in 'fdata'! The input of 'fdata' should be a functional data object produced by fd() function of fda package for univariate FADP, a list of functional data objects for multivariate FADP.", sep="\n" )
+FADPclust <- function(fdata, cluster = 2:10, method = "FADP1",
+                      proportion = NULL, f.cut = 0.15,
+                      pve = 0.90, stats = "silhouette"){
+  if( !inherits(fdata, "fd") & !inherits(fdata, "list") ){
+    stop( "Error in fdata! For univariate FADP: fdata should be a functional data object produced by fd() function of fda package, for multivariate FADP: a list of functional data objects.", sep="\n" )
   }
 
-  if( class(proportion) != "numeric" ){
-    stop( "Error in 'proportion'! The input of 'proportion' should be a number or numeric vector of numbers within the range [0,1].", sep="\n" )
-  }
+  if( is.null(proportion) & method == "FADP1" ){ proportion <- seq(0.1,1,0.1) }
 
-  if( min(proportion) < 0 | max(proportion) > 1){
-    stop( "Error in 'proportion'! The input value of 'proportion' should be between 0 and 1.", sep="\n" )
-  }
-
-  if( method != "FADP1" & method != "FADP2" ){
-    stop( "Error in 'method'! The input of 'method' is wrong, valid options of are 'FADP1' and 'FADP2'.", sep="\n" )
-  }
-
-  if( class(f.cut) != "numeric" ){
-    stop( "Error in 'f.cut'! The input of 'f.cut' should be a number between 0 and 1.", sep="\n" )
-  }
-
-  if( f.cut <= 0 | f.cut > 1){
-    stop( "Error in 'f.cut'! The input value of 'f.cut' should be between 0 and 1.", sep="\n" )
-  }
-
-  if( pve <= 0 | pve > 1){
-    stop( "Error in 'pve'! The input value of 'pve' should be between 0 and 1.", sep="\n" )
-  }
+  if( is.null(proportion) & method == "FADP2" ){  proportion <- seq(0.1,1,0.1) }
 
   ### Related functions ###
   knn_density1 <- function(distance, k){
     distmat    <- as.matrix(distance)
     n          <- nrow(distmat)
+    v.d        <- pi^(1/2) /gamma(1/2+1)
     r.k        <- apply(distmat, 1, sort)[k+1,]
-    den        <- k / (n * r.k)
+    den        <- k / (n * v.d * r.k)
     return(den)
   }
 
   knn_density2 <- function(score, k) {
-    n <- nrow(score); p <- ncol(score)
+    if (is.null(ncol(score)) == FALSE) {
+      n <- nrow(score); p <- ncol(score)
+    } else {n <- length(score); p <- 1}
     if (k >= n) {stop("k is not a reasonable and valid number!")}
     distance <- dist(score, method = "euclidean", upper = TRUE)
     distmat <- as.matrix(distance)
@@ -117,112 +108,128 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
     return(den)
   }
 
-  adp1 <- function(distance, clusters) {
+  assignment <- function(center, distance, den){
+    n.curve   <- nrow(distance)
+    assign <- rep(0, n.curve)
+    assign[center] <- 1:length(center)
+    assigned.index <- center
+    temp.index <- center
+    den.max <- which.max(den)
+    while (length(which(assign == 0)) != 0) {
+      loc <- rep(0, n.curve)
+      for (j in setdiff(1:n.curve, assigned.index)) {
+        if (j %in% den.max) {
+          nearest.neighbor <- center[which.min(distance[center, j])]
+        } else {
+          neighbors.index <- which(den[j] < den)
+          nearest.neighbor <- neighbors.index[which.min(distance[j, neighbors.index])]
+        }
+        loc[j] <- nearest.neighbor
+      }
+      for (l in 1:length(temp.index)) {
+        loc.index <- which(loc == temp.index[l])
+        assign[loc.index] = rep(assign[temp.index[l]], length(loc.index))
+      }
+      assigned.index <- which(assign != 0)
+      temp.index <- setdiff(assigned.index, temp.index)
+    }
+    return(assign)
+  }
+
+  adp1 <- function(distance, clusters){
     distance  <- as.matrix(distance)
     n.curve   <- nrow(distance)
     k.list    <- unique(ceiling(proportion * n.curve^(4/5)))
-    k.len     <- length(k.list)
-    clu.len   <- length(clusters)
-    den_del   <- array(data = NA, dim = c(2, n.curve, k.len))
-    sil_temp  <- matrix(data = NA, nrow = k.len, ncol = clu.len)
-    for (i in 1:k.len) {
-      den     <- knn_density1(distance = distance, k = k.list[i])
+
+    lapply.adp1 <- function(para.index){
+      k <- paralist[para.index,1]
+      m <- paralist[para.index,2]
+      den     <- knn_density1(distance = distance, k = k)
       del     <- c()
       den.max <- which(den == max(den), arr.ind = TRUE)
-      for (j in 1:n.curve) {
-        if (j %in% den.max) {
-          del[j] <- max(distance[j, ])
-        } else {
-          del[j] <- min(distance[j, which(den[j] < den)])
-        }
-      }
-      den_del[, , i] <- rbind(den, del)
-      ### Selection Criterion
-      clu.order <- 1
-      alpha     <- c()
-      for (m in clusters) {
-        if(round(n.curve*f.cut) < m){
-          alpha[clu.order] <- m/n.curve
-        }else{ alpha[clu.order] = f.cut }
-        den.index              <- which(den %in% sort(den, decreasing = T)[1:round(n.curve*alpha[clu.order])])
-        center.temp            <- which(del %in% sort(del[den.index], decreasing = T)[1:m])
-        result.temp            <- apply(distance[center.temp, ], 2, which.min)
-        sil                    <- summary(silhouette(result.temp, dmatrix = distance))[["avg.width"]]
-        sil_temp[i, clu.order] <- sil
-        clu.order              <- clu.order + 1
-      }
+      del <- mapply(function(j) if (j %in% den.max) { del[j] <- max(distance[j, ]) } else { del[j] <- min(distance[j, which(den[j] < den)]) }, 1:n.curve)
+      if(round(n.curve*f.cut) < m){
+        alpha <- m/n.curve
+      }else{ alpha = f.cut }
+      den.index              <- which(den %in% sort(den, decreasing = T)[1:round(n.curve*alpha)])
+      center.temp            <- which(del %in% sort(del[den.index], decreasing = T)[1:m])
+      result.temp            <- assignment(center = center.temp, distance = distance, den = den)
+      stats.temp             <- cluster.stats(d = distance, clustering = result.temp)
+      if(stats == "silhouette"){ evaluation <- stats.temp$avg.silwidth }
+      if(stats == "Dunn"){ evaluation <- stats.temp$dunn2 }
+      if(stats == "CH"){ evaluation <- stats.temp$ch }
+      return(list(evaluation = evaluation, clustering = result.temp, density = den, delta = del, center = center.temp))
     }
-    index          <- which(sil_temp == sil_temp[which.max(sil_temp)], arr.ind = T)
-    index          <- index[1, ]
-    den            <- den_del[1, , index[1]]
-    del            <- den_del[2, , index[1]]
-    nclust         <- clusters[index[2]]
-    if(round(n.curve*f.cut) < nclust){
-      alpha_opt <- nclust/n.curve
-    }else{ alpha_opt = f.cut }
-    den.index      <- which( den %in% sort(den, decreasing = T)[1:round(n.curve*alpha_opt)] )
-    center.index   <- which(del %in% sort(del[den.index], decreasing = T)[1:nclust])
-    res_clustering <- as.numeric(apply(distance[center.index, ], 2, which.min))
-    result         <- list(length(unique(center.index)), round(k.list[index[1]]), method, res_clustering, den, del, center.index, max(sil_temp))
-    names(result)  <- c("nclust", "para","method", "clust", "density", "delta", "center", "silhouette")
+
+    paralist <- expand.grid(k.list, clusters)
+    task.len <- nrow(paralist)
+    result.temp <- lapply(1:task.len, lapply.adp1)
+    evaluation.list <- unlist(lapply(1:task.len, function(l) result.temp[[l]]$evaluation))
+    selected.index <- which.max(evaluation.list)
+    k.selected <- paralist[selected.index, 1]
+    nclust.selected <- paralist[selected.index, 2]
+    clustering.selected <- result.temp[[selected.index]]$clustering
+    density.selected <- result.temp[[selected.index]]$density
+    delta.selected <- result.temp[[selected.index]]$delta
+    center.selected <- result.temp[[selected.index]]$center
+    result         <- list(nclust.selected, proportion[which(k.list == k.selected)],
+                           method, clustering.selected, density.selected, delta.selected, center.selected, max(evaluation.list))
+    #result         <- list(nclust.selected, round(k.selected),
+    #                       method, clustering.selected, density.selected, delta.selected, center.selected, max(evaluation.list))
+    names(result)  <- c("nclust", "para","method", "clust", "density", "delta", "center", "stats")
     return(result)
   }
 
-  adp2 <- function(distance, clusters, score) {
+  adp2 <- function(distance, clusters, score){
     distance  <- as.matrix(distance)
     n.curve   <- nrow(distance)
     k.list    <- unique(ceiling(proportion * n.curve^(4/5)))
-    k.len     <- length(k.list)
-    clu.len   <- length(clusters)
-    den_del   <- array(data = NA, dim = c(2, n.curve, k.len))
-    sil_temp  <- matrix(data = NA, nrow = k.len, ncol = clu.len)
-    for (i in 1:k.len) {
-      den     <- knn_density2(score = score, k = k.list[i])
+
+    lapply.adp2 <- function(para.index){
+      k <- paralist[para.index,1]
+      m <- paralist[para.index,2]
+
+      den     <- knn_density2(score = score, k = k)
       del     <- c()
       den.max <- which(den == max(den), arr.ind = TRUE)
-      for (j in 1:n.curve) {
-        if (j %in% den.max) {
-          del[j] <- max(distance[j, ])
-        } else {
-          del[j] <- min(distance[j, which(den[j] < den)])
-        }
-      }
-      den_del[, , i] <- rbind(den, del)
-      ### Selection Criterion
-      clu.order <- 1
-      alpha     <- c()
-      for (m in clusters) {
-        if(round(n.curve*f.cut) < m){
-          alpha[clu.order] <- m/n.curve
-        }else{ alpha[clu.order] = f.cut }
-        den.index              <- which(den %in% sort(den, decreasing = T)[1:round(n.curve*alpha[clu.order])])
-        center.temp            <- which(del %in% sort(del[den.index], decreasing = T)[1:m])
-        result.temp            <- apply(distance[center.temp, ], 2, which.min)
-        sil                    <- summary(silhouette(result.temp, dmatrix = distance))[["avg.width"]]
-        sil_temp[i, clu.order] <- sil
-        clu.order              <- clu.order + 1
-      }
+      del <- mapply(function(j) if (j %in% den.max) { del[j] <- max(distance[j, ]) } else { del[j] <- min(distance[j, which(den[j] < den)]) }, 1:n.curve)
+      if(round(n.curve*f.cut) < m){
+        alpha <- m/n.curve
+      }else{ alpha = f.cut }
+      den.index              <- which(den %in% sort(den, decreasing = T)[1:round(n.curve*alpha)])
+      center.temp            <- which(del %in% sort(del[den.index], decreasing = T)[1:m])
+      result.temp            <- assignment(center = center.temp, distance = distance, den = den)
+      stats.temp             <- cluster.stats(d = distance, clustering = result.temp)
+      if(stats == "silhouette"){ evaluation <- stats.temp$avg.silwidth }
+      if(stats == "Dunn"){ evaluation <- stats.temp$dunn2 }
+      if(stats == "CH"){ evaluation <- stats.temp$ch }
+      return(list(evaluation = evaluation, clustering = result.temp, density = den, delta = del, center = center.temp))
     }
-    index          <- which(sil_temp == sil_temp[which.max(sil_temp)], arr.ind = T)
-    index          <- index[1, ]
-    den            <- den_del[1, , index[1]]
-    del            <- den_del[2, , index[1]]
-    nclust         <- clusters[index[2]]
-    if(round(n.curve*f.cut) < nclust){
-      alpha_opt <- nclust/n.curve
-    }else{ alpha_opt = f.cut }
-    den.index      <- which( den %in% sort(den, decreasing = T)[1:round(n.curve*alpha_opt)] )
-    center.index   <- which(del %in% sort(del[den.index], decreasing = T)[1:nclust])
-    res_clustering <- as.numeric(apply(distance[center.index, ], 2, which.min))
-    eta.k.opt      <- apply(distance, 1, sort)[k.list[index[1]] + 1, ]
+    paralist <- expand.grid(k.list, clusters)
+    task.len <- nrow(paralist)
+    result.temp <- lapply(1:task.len, lapply.adp2)
+
+    evaluation.list <- unlist(lapply(1:task.len, function(l) result.temp[[l]]$evaluation))
+    selected.index <- which.max(evaluation.list)
+    k.selected <- paralist[selected.index, 1]
+    nclust.selected <- paralist[selected.index, 2]
+    clustering.selected <- result.temp[[selected.index]]$clustering
+    density.selected <- result.temp[[selected.index]]$density
+    delta.selected <- result.temp[[selected.index]]$delta
+    center.selected <- result.temp[[selected.index]]$center
+    eta.k.opt      <- apply(distance, 1, sort)[k.selected + 1, ]
     mu.k.opt       <- mean(eta.k.opt)
     h.opt          <- mu.k.opt + sqrt(sum((eta.k.opt - mu.k.opt)^2)/(n.curve - 1))
-    result         <- list(length(unique(center.index)), round(k.list[index[1]]), method, res_clustering, den, del, center.index, max(sil_temp))
-    names(result)  <- c("nclust", "para", "method", "clust", "density", "delta", "center", "silhouette")
+
+    result         <- list(nclust.selected, proportion[which(k.list == k.selected)],
+                           method, clustering.selected, density.selected, delta.selected, center.selected, max(evaluation.list))
+    #result         <- list(nclust.selected, h.opt,
+    #                       method, clustering.selected, density.selected, delta.selected, center.selected, max(evaluation.list))
+    names(result)  <- c("nclust", "para","method", "clust", "density", "delta", "center", "stats")
     return(result)
   }
 
-  if( class(fdata) == "fd" & method == "FADP1" ){
+  if( inherits(fdata, "fd") & method == "FADP1" ){
     nbasis      <- fdata$basis$nbasis
     type        <- fdata$basis$type
     distance_L2 <- semimetric.basis(fdata1 = fdata, fdata2 = fdata, nderiv = 0, type.basis1 = type,
@@ -230,7 +237,7 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
     result      <- adp1(distance = distance_L2, clusters = cluster)
   }
 
-  if( class(fdata) == "list" & method == "FADP1" ){
+  if( inherits(fdata, "list") & method == "FADP1" ){
     basis    <- fdata[[1]]$basis
     nbasis   <- fdata[[1]]$basis$nbasis
     rangeval <- fdata[[1]]$basis$rangeval
@@ -238,6 +245,7 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
     n.curve  <- ncol(fdata[[1]]$coefs)
     dot      <- nbasis*10
     t        <- seq(rangeval[1], rangeval[2], by = (rangeval[2] - rangeval[1])/(dot - 1))
+    # dot=nbasis+1
     data     <- array(0, dim = c(n.curve, dot, p))
     for (i in 1:p) {
       data[, , i] <- t(eval.fd(evalarg = t, fdobj = fdata[[i]]))
@@ -255,7 +263,7 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
     lam <- matrix(0, p, dot)  #eigenvalues
     a   <- array(data = 0, dim = c(p, dot, p))  #eigenvectors
     for (i in 1:dot) {
-        xir      <- matrix(0, p, n.curve)
+      xir      <- matrix(0, p, n.curve)
       for (r in 1:p) {
         xir[r, ] <- data[, i, r]
       }
@@ -303,7 +311,7 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
     }
 
     #################### Step 3: calculate the variability explained
-    v  <- lam[1, ] + lam[2, ]
+    v  <- apply(lam, 2, sum)#lam[1, ] + lam[2, ]
     phi <- c()
     # calculate by phi_2
     for (k in 1:p) {
@@ -325,7 +333,7 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
 
     #################### Step 4:calculate the r-th principal components
     component <- function(r) {
-        bind.matrix <- data[, , 1]
+      bind.matrix <- data[, , 1]
       for (m in 2:p) {
         bind.matrix <- cbind(bind.matrix, data[, , m])
       }
@@ -350,8 +358,8 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
       comp                <- component(r)
       compfd              <- Data2fd(argvals = t, y = t(comp), basisobj = basis)
       L2_distance         <- semimetric.basis(fdata1 = compfd, fdata2 = compfd, nderiv = 0,
-                                      type.basis1 = "bspline", nbasis1 = nbasis, type.basis2 = "bspline",
-                                      nbasis2 = nbasis)
+                                              type.basis1 = "bspline", nbasis1 = nbasis, type.basis2 = "bspline",
+                                              nbasis2 = nbasis)
       comp_list[[r]]      <- compfd
       distance_list2[[r]] <- L2_distance
       distance_L2         <- distance_L2 + distance_list2[[r]]^2 * beta
@@ -360,33 +368,34 @@ FADPclust <- function(fdata, cluster = 2:10, method = "FADP1", proportion = seq(
     result   <- adp1(distance = distance, clusters = cluster)
   }
 
-  if( class(fdata) == "fd" & method == "FADP2" ){
+  if( inherits(fdata, "fd") & method == "FADP2" ){
     nbasis   <- fdata$basis$nbasis
     rangeval <- fdata$basis$rangeval
     dot      <- nbasis*10
     t        <- seq(rangeval[1], rangeval[2], by = (rangeval[2] - rangeval[1])/(dot - 1))
     fundata  <- funData(argvals = t, X = t(eval.fd(evalarg = t, fdobj = fdata)))
-    pca  <- PACE(fundata, pve = pve)
+    pca  <- PACE(fundata, pve=pve)
     score       <- pca$scores
     distance_L2 <- dist(score, method = "euclidean", upper = TRUE)
 
     result      <- adp2(distance = distance_L2, clusters = cluster, score = score)
   }
 
-  if( class(fdata) == "list" & method == "FADP2" ){
+  if( inherits(fdata, "list") & method == "FADP2" ){
     nbasis   <- fdata[[1]]$basis$nbasis
     rangeval <- fdata[[1]]$basis$rangeval
     dot      <- nbasis*10
     t        <- seq(rangeval[1], rangeval[2], by = (rangeval[2] - rangeval[1])/(dot - 1))
     fundata  <- list()
     type     <- list()
+    comp_num <- c()
     for(i in 1:length(fdata)){
       fundata[[i]]  <- funData(argvals = t, X = t(eval.fd(evalarg = t, fdobj = fdata[[i]])) )
       type[[i]]     <- list(type = "uFPCA")
+      comp_num[i]   <- PACE(fundata[[i]], pve=pve)$npc
     }
-    comp_num   <- PACE(fundata[[1]], pve = pve)$npc
     fundata <- multiFunData(fundata)
-    pca  <- MFPCA(mFData = fundata, M = comp_num, uniExpansions = type)
+    pca  <- MFPCA(mFData = fundata, M = max(comp_num), uniExpansions = type)
     score       <- pca$scores
     distance_L2 <- dist(score, method = "euclidean", upper = TRUE)
 
